"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const spec_1 = require("@specron/spec");
const types_1 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertBurnProxy = yield ctx.deploy({
        src: './build/xcert-burn-proxy.json',
        contract: 'XcertBurnProxy',
    });
    ctx.set('xcertBurnProxy', xcertBurnProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertBurnProxy = ctx.get('xcertBurnProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield xcertBurnProxy.instance.methods.grantAbilities(bob, types_1.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertBurnProxy.instance.methods.isAble(bob, types_1.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertBurnProxy = ctx.get('xcertBurnProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield xcertBurnProxy.instance.methods.grantAbilities(bob, types_1.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield xcertBurnProxy.instance.methods.revokeAbilities(bob, types_1.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertBurnProxy.instance.methods.isAble(bob, types_1.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('destroys an Xcert', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertBurnProxy = ctx.get('xcertBurnProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const imprint = '0x16ba121a04d5ad3b885f09af5c6c08b68766ce43d990f8420dc97f6ff485cb37';
    yield xcertBurnProxy.instance.methods.grantAbilities(bob, types_1.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x9d118770']],
    });
    yield cat.instance.methods.create(jane, 1, imprint).send({ from: owner });
    yield cat.instance.methods.setApprovalForAll(xcertBurnProxy.receipt._address, true).send({ from: jane });
    yield xcertBurnProxy.instance.methods.destroy(cat.receipt._address, 1).send({ from: bob });
    yield ctx.reverts(() => cat.instance.methods.ownerOf(1).call());
}));
spec.test('fails if destroy is triggered by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertBurnProxy = ctx.get('xcertBurnProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const imprint = '0x16ba121a04d5ad3b885f09af5c6c08b68766ce43d990f8420dc97f6ff485cb37';
    yield xcertBurnProxy.instance.methods.grantAbilities(bob, types_1.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x9d118770']],
    });
    yield cat.instance.methods.create(jane, 1, imprint).send({ from: owner });
    yield ctx.reverts(() => xcertBurnProxy.instance.methods.destroy(cat.receipt._address, 1).send({ from: bob }));
}));
exports.default = spec;
//# sourceMappingURL=burn.test.js.map