"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ethereum_utils_1 = require("@0xcert/ethereum-utils");
const types_1 = require("@0xcert/ethereum-xcert-contracts/src/core/types");
const spec_1 = require("@specron/spec");
const types_2 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = yield ctx.deploy({
        src: './build/xcert-create-proxy.json',
        contract: 'XcertCreateProxy',
    });
    ctx.set('xcertProxy', xcertProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = ctx.get('xcertProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield xcertProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertProxy.instance.methods.isAble(bob, types_2.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = ctx.get('xcertProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield xcertProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield xcertProxy.instance.methods.revokeAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertProxy.instance.methods.isAble(bob, types_2.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('creates an Xcert', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = ctx.get('xcertProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    yield xcertProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', []],
    });
    yield cat.instance.methods.grantAbilities(xcertProxy.receipt._address, types_1.XcertAbilities.CREATE_ASSET).send({ from: owner });
    yield xcertProxy.instance.methods.create(cat.receipt._address, jane, 1, '0x0').send({ from: bob });
    const newOwner = yield cat.instance.methods.ownerOf(1).call();
    ctx.is(newOwner, jane);
}));
spec.test('creates an Xcert with high ID', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = ctx.get('xcertProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    yield xcertProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', []],
    });
    yield cat.instance.methods.grantAbilities(xcertProxy.receipt._address, types_1.XcertAbilities.CREATE_ASSET).send({ from: owner });
    yield xcertProxy.instance.methods.create(cat.receipt._address, jane, ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE'), '0x0').send({ from: bob });
    const newOwner = yield cat.instance.methods.ownerOf(ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE')).call();
    ctx.is(newOwner, jane);
}));
spec.test('fails if create is triggered by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertProxy = ctx.get('xcertProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', []],
    });
    yield cat.instance.methods.grantAbilities(xcertProxy.receipt._address, types_1.XcertAbilities.CREATE_ASSET).send({ from: owner });
    yield ctx.reverts(() => xcertProxy.instance.methods.create(cat.receipt._address, jane, 1, '0x0').send({ from: bob }), '017001');
}));
exports.default = spec;
//# sourceMappingURL=create.test.js.map