"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@0xcert/ethereum-xcert-contracts/src/core/types");
const spec_1 = require("@specron/spec");
const types_2 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = yield ctx.deploy({
        src: './build/abilitable-manage-proxy.json',
        contract: 'AbilitableManageProxy',
    });
    ctx.set('abilitableManageProxy', abilitableManageProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = ctx.get('abilitableManageProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield abilitableManageProxy.instance.methods.grantAbilities(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield abilitableManageProxy.instance.methods.isAble(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = ctx.get('abilitableManageProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield abilitableManageProxy.instance.methods.grantAbilities(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield abilitableManageProxy.instance.methods.revokeAbilities(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield abilitableManageProxy.instance.methods.isAble(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('sets abilities', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = ctx.get('abilitableManageProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    yield abilitableManageProxy.instance.methods.grantAbilities(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x0d04c3b8']],
    });
    yield cat.instance.methods.grantAbilities(abilitableManageProxy.receipt._address, types_1.XcertAbilities.MANAGE_ABILITIES).send({ from: owner });
    yield abilitableManageProxy.instance.methods.set(cat.receipt._address, jane, types_1.XcertAbilities.TOGGLE_TRANSFERS).send({ from: bob });
    const hasToggleTransferAbility = yield cat.instance.methods.isAble(jane, types_1.XcertAbilities.TOGGLE_TRANSFERS).call();
    ctx.true(hasToggleTransferAbility);
}));
spec.test('removes self manage abilities', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = ctx.get('abilitableManageProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield abilitableManageProxy.instance.methods.grantAbilities(bob, types_2.AbilitableManageProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x0d04c3b8']],
    });
    yield cat.instance.methods.grantAbilities(abilitableManageProxy.receipt._address, types_1.XcertAbilities.MANAGE_ABILITIES).send({ from: owner });
    yield abilitableManageProxy.instance.methods.set(cat.receipt._address, abilitableManageProxy.receipt._address, 0).send({ from: bob });
    const hasManageAbility = yield cat.instance.methods.isAble(abilitableManageProxy.receipt._address, types_1.XcertAbilities.MANAGE_ABILITIES).call();
    ctx.false(hasManageAbility);
}));
spec.test('fails if set abilities is called by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const abilitableManageProxy = ctx.get('abilitableManageProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x0d04c3b8']],
    });
    yield cat.instance.methods.grantAbilities(abilitableManageProxy.receipt._address, types_1.XcertAbilities.MANAGE_ABILITIES).send({ from: owner });
    yield ctx.reverts(() => abilitableManageProxy.instance.methods.set(cat.receipt._address, jane, types_1.XcertAbilities.TOGGLE_TRANSFERS).send({ from: bob }));
}));
exports.default = spec;
//# sourceMappingURL=manage.test.js.map