"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ethereum_utils_1 = require("@0xcert/ethereum-utils");
const spec_1 = require("@specron/spec");
const types_1 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
    ctx.set('sara', accounts[3]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = yield ctx.deploy({
        src: './build/nftoken-transfer-proxy.json',
        contract: 'NFTokenTransferProxy',
    });
    ctx.set('nftProxy', nftProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = ctx.get('nftProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield nftProxy.instance.methods.grantAbilities(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield nftProxy.instance.methods.isAble(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = ctx.get('nftProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield nftProxy.instance.methods.grantAbilities(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield nftProxy.instance.methods.revokeAbilities(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield nftProxy.instance.methods.isAble(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('transfers an NFT', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = ctx.get('nftProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const sara = ctx.get('sara');
    yield nftProxy.instance.methods.grantAbilities(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-erc721-contracts/build/nf-token-metadata-enumerable-mock.json',
        contract: 'NFTokenMetadataEnumerableMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json'],
    });
    yield cat.instance.methods
        .create(jane, 1)
        .send({
        from: owner,
        gas: 4000000,
    });
    yield cat.instance.methods.approve(nftProxy.receipt._address, 1).send({ from: jane });
    yield nftProxy.instance.methods.execute(cat.receipt._address, jane, sara, 1).send({ from: bob });
    const newOwner = yield cat.instance.methods.ownerOf(1).call();
    ctx.is(newOwner, sara);
}));
spec.test('transfers an NFT with high ID', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = ctx.get('nftProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const sara = ctx.get('sara');
    yield nftProxy.instance.methods.grantAbilities(bob, types_1.NFTokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-erc721-contracts/build/nf-token-metadata-enumerable-mock.json',
        contract: 'NFTokenMetadataEnumerableMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json'],
    });
    yield cat.instance.methods
        .create(jane, ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE'))
        .send({
        from: owner,
        gas: 4000000,
    });
    yield cat.instance.methods.approve(nftProxy.receipt._address, ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE')).send({ from: jane });
    yield nftProxy.instance.methods.execute(cat.receipt._address, jane, sara, ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE')).send({ from: bob });
    const newOwner = yield cat.instance.methods.ownerOf(ethereum_utils_1.bigNumberify('0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE')).call();
    ctx.is(newOwner, sara);
}));
spec.test('fails if transfer is triggered by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const nftProxy = ctx.get('nftProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const sara = ctx.get('sara');
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-erc721-contracts/build/nf-token-metadata-enumerable-mock.json',
        contract: 'NFTokenMetadataEnumerableMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json'],
    });
    yield cat.instance.methods
        .create(jane, 1)
        .send({
        from: owner,
        gas: 4000000,
    });
    yield cat.instance.methods.approve(nftProxy.receipt._address, 1).send({ from: jane });
    yield ctx.reverts(() => nftProxy.instance.methods.execute(cat.receipt._address, jane, sara, 1).send({ from: bob }), '017001');
}));
exports.default = spec;
//# sourceMappingURL=nftokens.test.js.map