"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const spec_1 = require("@specron/spec");
const types_1 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenProxy = yield ctx.deploy({
        src: './build/token-transfer-proxy.json',
        contract: 'TokenTransferProxy',
    });
    ctx.set('tokenProxy', tokenProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenProxy = ctx.get('tokenProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield tokenProxy.instance.methods.grantAbilities(bob, types_1.TokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield tokenProxy.instance.methods.isAble(bob, types_1.TokenTransferProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenProxy = ctx.get('tokenProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield tokenProxy.instance.methods.grantAbilities(bob, types_1.TokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield tokenProxy.instance.methods.revokeAbilities(bob, types_1.TokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield tokenProxy.instance.methods.isAble(bob, types_1.TokenTransferProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('transfers tokens', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenProxy = ctx.get('tokenProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    yield tokenProxy.instance.methods.grantAbilities(bob, types_1.TokenTransferProxyAbilities.EXECUTE).send({ from: owner });
    const token = yield ctx.deploy({
        src: '@0xcert/ethereum-erc20-contracts/build/token-mock.json',
        contract: 'TokenMock',
        args: ['ERC20', 'ERC', 18, '300000000000000000000000000'],
    });
    yield token.instance.methods.approve(tokenProxy.receipt._address, 1000).send({ from: owner });
    yield tokenProxy.instance.methods.execute(token.receipt._address, owner, jane, 1000).send({ from: bob });
    const balance = yield token.instance.methods.balanceOf(jane).call();
    ctx.is(balance, '1000');
}));
spec.test('fails if transfer is triggered by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenProxy = ctx.get('tokenProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const token = yield ctx.deploy({
        src: '@0xcert/ethereum-erc20-contracts/build/token-mock.json',
        contract: 'TokenMock',
        args: ['ERC20', 'ERC', 18, '300000000000000000000000000'],
    });
    yield token.instance.methods.approve(tokenProxy.receipt._address, 1000).send({ from: owner });
    yield ctx.reverts(() => tokenProxy.instance.methods.execute(token.receipt._address, owner, jane, 1000).send({ from: bob }), '017001');
}));
exports.default = spec;
//# sourceMappingURL=tokens.test.js.map