"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@0xcert/ethereum-xcert-contracts/src/core/types");
const spec_1 = require("@specron/spec");
const types_2 = require("../core/types");
const spec = new spec_1.Spec();
spec.before((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield ctx.web3.eth.getAccounts();
    ctx.set('owner', accounts[0]);
    ctx.set('bob', accounts[1]);
    ctx.set('jane', accounts[2]);
}));
spec.beforeEach((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertUpdateProxy = yield ctx.deploy({
        src: './build/xcert-update-proxy.json',
        contract: 'XcertUpdateProxy',
    });
    ctx.set('xcertUpdateProxy', xcertUpdateProxy);
}));
spec.test('adds authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertUpdateProxy = ctx.get('xcertUpdateProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const logs = yield xcertUpdateProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertUpdateProxy.instance.methods.isAble(bob, types_2.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, true);
}));
spec.test('removes authorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertUpdateProxy = ctx.get('xcertUpdateProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    yield xcertUpdateProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const logs = yield xcertUpdateProxy.instance.methods.revokeAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    ctx.not(logs.events.SetAbilities, undefined);
    const bobHasAbilityToExecute = yield xcertUpdateProxy.instance.methods.isAble(bob, types_2.XcertCreateProxyAbilities.EXECUTE).call();
    ctx.is(bobHasAbilityToExecute, false);
}));
spec.test('updates an Xcert', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertUpdateProxy = ctx.get('xcertUpdateProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const imprint = '0x16ba121a04d5ad3b885f09af5c6c08b68766ce43d990f8420dc97f6ff485cb37';
    const newURIIntegrityDigest = '0x6108466288093cf38a5b155b015dc556cb23dd48e3bf4739aef4932541db8eea';
    yield xcertUpdateProxy.instance.methods.grantAbilities(bob, types_2.XcertCreateProxyAbilities.EXECUTE).send({ from: owner });
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x0d04c3b8']],
    });
    yield cat.instance.methods.grantAbilities(xcertUpdateProxy.receipt._address, types_1.XcertAbilities.UPDATE_ASSET_IMPRINT).send({ from: owner });
    yield cat.instance.methods.create(jane, 1, imprint).send({ from: owner });
    let token1URIIntegrityDigest = yield cat.instance.methods.tokenURIIntegrity(1).call();
    ctx.is(token1URIIntegrityDigest.digest, imprint);
    yield xcertUpdateProxy.instance.methods.update(cat.receipt._address, 1, newURIIntegrityDigest).send({ from: bob });
    token1URIIntegrityDigest = yield cat.instance.methods.tokenURIIntegrity(1).call();
    ctx.is(token1URIIntegrityDigest.digest, newURIIntegrityDigest);
}));
spec.test('fails if update is triggered by an unauthorized address', (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const xcertUpdateProxy = ctx.get('xcertUpdateProxy');
    const owner = ctx.get('owner');
    const bob = ctx.get('bob');
    const jane = ctx.get('jane');
    const imprint = '0x16ba121a04d5ad3b885f09af5c6c08b68766ce43d990f8420dc97f6ff485cb37';
    const newURIIntegrityDigest = '0x6108466288093cf38a5b155b015dc556cb23dd48e3bf4739aef4932541db8eea';
    const cat = yield ctx.deploy({
        src: '@0xcert/ethereum-xcert-contracts/build/xcert-mock.json',
        contract: 'XcertMock',
        args: ['cat', 'CAT', 'https://0xcert.org/', '.json', '0xa65de9e6', ['0x0d04c3b8']],
    });
    yield cat.instance.methods.grantAbilities(xcertUpdateProxy.receipt._address, types_1.XcertAbilities.UPDATE_ASSET_IMPRINT).send({ from: owner });
    yield cat.instance.methods.create(jane, 1, imprint);
    yield ctx.reverts(() => xcertUpdateProxy.instance.methods.update(cat.receipt._address, 1, newURIIntegrityDigest).send({ from: bob }), '017001');
}));
exports.default = spec;
//# sourceMappingURL=update.test.js.map